/*
 *  linux/include/linux/hfsplus_fs.h
 *
 * Copyright (C) 1999
 * Brad Boyer (flar@pants.nu)
 *
 */

#ifndef _LINUX_HFSPLUS_FS_H
#define _LINUX_HFSPLUS_FS_H

#include <linux/fs.h>

#include <asm/unaligned.h>

#include <linux/hfsplus_raw.h>
#include <linux/hfsplus_fs_sb.h>

typedef  __u8 hfsp_u8;
typedef __u16 hfsp_u16;
typedef __u32 hfsp_u32;
typedef __u64 hfsp_u64;
typedef __u32 hfsp_cnid;

/* Runtime config options */
#define HFSPLUS_CASE_ASIS      0
#define HFSPLUS_CASE_LOWER     1

#define HFSPLUS_FORK_RAW       0
#define HFSPLUS_FORK_CAP       1
#define HFSPLUS_FORK_DOUBLE    2
#define HFSPLUS_FORK_NETATALK  3

#define HFSPLUS_NAMES_TRIVIAL  0
#define HFSPLUS_NAMES_CAP      1
#define HFSPLUS_NAMES_NETATALK 2
#define HFSPLUS_NAMES_7BIT     3

#define HFSPLUS_DEF_CR_TYPE    0x3F3F3F3F  /* '????' */

#define HFSPLUS_TYPE_DATA 0x00
#define HFSPLUS_TYPE_RSRC 0xFF

typedef int (*btree_keycmp)(hfsplus_btree_key *, hfsplus_btree_key *);

/* An HFS+ BTree held in memory */
typedef struct hfsplus_btree {
	struct super_block *sb;
	btree_keycmp        keycmp;

	hfsp_u32            cnid;
	hfsp_u32            root;
	hfsp_u32            node_count;
	hfsp_u32            free_nodes;
	hfsp_u32            attributes;

	hfsp_u16            node_size;
	hfsp_u16            max_key_len;
	hfsp_u16            depth;
} hfsplus_btree;

struct page;

/* An HFS+ BTree node in memory */
typedef struct {
	struct hfsplus_btree *tree;
	struct inode         *inode;

	hfsp_u32              prev;
	hfsp_u32              this;
	hfsp_u32              next;

	hfsp_u16              num_recs;
	hfsp_u8               kind;
	hfsp_u8               height;
} hfsplus_bnode;

/* An iterator for walking an HFS+ BTree */
typedef struct {
	hfsplus_btree *tree;
	hfsplus_bnode *cached;
	hfsp_u32 node;
	hfsp_u16 rec;
} hfsplus_btiter;

/*
 * Functions in any *.c used in other files
 */

/* bfind.c */
int hfsplus_btfind(hfsplus_btree *, hfsplus_btree_key *, void *, int *);
int hfsplus_btfind_exact(hfsplus_btree *, hfsplus_btree_key *, void *, int *);
int hfsplus_btiter_find(hfsplus_btiter *, hfsplus_btree_key *);

/* bnode.c */
hfsplus_bnode *hfsplus_get_bnode(hfsplus_btree *, hfsp_u32);
void hfsplus_put_bnode(hfsplus_bnode *);
hfsp_u16 hfsplus_bnode_copybytes(hfsplus_bnode *, void *, hfsp_u16, hfsp_u16);

/* brec.c */
hfsp_u16 hfsplus_brec_off(hfsplus_bnode *, hfsp_u16);
hfsp_u16 hfsplus_brec_len(hfsplus_bnode *, hfsp_u16);
hfsp_u16 hfsplus_brec_lenoff(hfsplus_bnode *, hfsp_u16, hfsp_u16 *);
hfsp_u16 hfsplus_brec_data(hfsplus_bnode *, hfsp_u16, char *, hfsp_u16);
hfsp_u16 hfsplus_brec_keylen(hfsplus_bnode *, hfsp_u16);
hfsp_u16 hfsplus_brec_key(hfsplus_bnode *, hfsp_u16, void *, hfsp_u16);

/* btiter.c */
void hfsplus_btiter_setup(hfsplus_btiter *, hfsplus_btree *);
int hfsplus_btiter_lock(hfsplus_btiter *);
int hfsplus_btiter_unlock(hfsplus_btiter *);
int hfsplus_btiter_move(hfsplus_btiter *, int);
int hfsplus_btiter_get(hfsplus_btiter *, hfsplus_btree_key *, void *, int *);

/* btree.c */
hfsplus_btree *hfsplus_open_btree(struct super_block *, hfsp_cnid);
void hfsplus_close_btree(struct hfsplus_btree *);

/* catalog.c */
int hfsplus_cmp_cat_key(hfsplus_btree_key *, hfsplus_btree_key *);
int hfsplus_fill_cat_key(hfsplus_btree_key *, hfsp_cnid, char *, int);
int hfsplus_find_cat(struct super_block *, unsigned long, hfsplus_cat_entry *);

/* extents.c */
int hfsplus_cmp_ext_key(hfsplus_btree_key *, hfsplus_btree_key *);
void hfsplus_fill_ext_key(hfsplus_btree_key *, hfsp_u32, hfsp_u32, hfsp_u8);
int hfsplus_get_block(struct inode *, long, struct buffer_head *, int);

/* inode.c */
int hfsplus_cat_fill_inode(struct inode *, hfsplus_cat_entry *);

/* options.c */
int parse_options(char *, struct hfsplus_sb_info *);
void fill_defaults(struct hfsplus_sb_info *);
void fill_current(struct hfsplus_sb_info *,  struct hfsplus_sb_info *);

/* tables.c */
extern hfsp_u16 case_fold_table[];

/* unicode.c */
int hfsplus_unistrcmp(hfsplus_unistr *, hfsplus_unistr *);
int hfsplus_uni2asc(hfsplus_unistr *, char *, int *);
int hfsplus_asc2uni(hfsplus_unistr *, char *, int);

/* wrapper.c */
int hfsplus_read_wrapper(struct super_block *);

/* access macros */
#define HFSPLUS_SB(super)        ((super)->u.hfsplus_sb)
#define HFSPLUS_I(inode)         ((inode)->u.hfsplus_i)

#define HFSPLUS_IS_DATA(inode)   (1)
#define HFSPLUS_IS_RSRC(inode)   (0)

#define hfsp_get_hs(addr)        ntohs(*((hfsp_u16 *)(addr)))
#define hfsp_get_hl(addr)        ntohl(get_unaligned((hfsp_u32 *)(addr)))
#define hfsp_get_ahl(addr)       ntohl(*((hfsp_u32 *)(addr)))
#define hfsp_get_hll(addr)       __be64_to_cpu(get_unaligned((hfsp_u64 *)(addr)))
#define hfsp_get_ahll(addr)      __be64_to_cpu(*((hfsp_u64 *)(addr)))
#define hfsp_put_hs(val, addr)   ((void)(*((hfsp_u16 *)(addr)) = ntohs(val)))
#define hfsp_put_hl(val, addr)   put_unaligned(htonl(val), (hfsp_u32 *)(addr))
#define hfsp_put_ahl(val, addr)  ((void)(*((hfsp_u32 *)(addr)) = ntohl(val)))
#define hfsp_put_hll(val, addr)  put_unaligned(__cpu_to_be64(val), \
					       (hfsp_u64 *)(addr))
#define hfsp_put_ahll(val, addr) ((void)(*((hfsp_u64 *)(addr)) = \
				  __cpu_to_be64(val)))

/* time macros */
#define hfsp_mt2ut(t)          (hfsp_get_hl(t) - 2082844800U)

#endif
