/*
 *  linux/include/linux/hfsplus_raw.h
 *
 * Copyright (C) 1999
 * Brad Boyer (flar@pants.nu)
 *
 * Format of structures on disk
 * Information taken from Apple Technote #1150 (HFS Plus Volume Format)
 *
 */

#ifndef _LINUX_HFSPLUS_RAW_H
#define _LINUX_HFSPLUS_RAW_H

#include <asm/types.h>

/* Some constants */
#define HFSPLUS_SECTOR_SIZE        512
#define HFSPLUS_VOLHEAD_SECTOR       2
#define HFSPLUS_VOLHEAD_SIG     0x482b
#define HFSPLUS_SUPER_MAGIC     0x482b
#define HFSPLUS_CURRENT_VERSION      4

#define HFSP_WRAP_MAGIC         0x4244
#define HFSP_WRAP_ATTRIB_SLOCK  0x8000
#define HFSP_WRAP_ATTRIB_SPARED 0x0200

#define HFSP_WRAPOFF_SIG          0x00
#define HFSP_WRAPOFF_ATTRIB       0x0A
#define HFSP_WRAPOFF_ABLKSIZE     0x14
#define HFSP_WRAPOFF_ABLKSTART    0x1C
#define HFSP_WRAPOFF_EMBEDSIG     0x7C
#define HFSP_WRAPOFF_EMBEDEXT     0x7E

/* Structures used on disk */

typedef unsigned char hfsp_byte;
typedef unsigned char hfsp_word[2];
typedef unsigned char hfsp_lword[4];
typedef unsigned char hfsp_llword[8];

typedef hfsp_lword hfsplus_cnid;
typedef hfsp_word hfsplus_unichr;

/* A "string" as used in filenames, etc. */
typedef struct {
	hfsp_word          length;
	hfsplus_unichr     unicode[255];
} hfsplus_unistr;

#define HFSPLUS_MAX_STRLEN 255

/* POSIX permissions */
typedef struct {
	hfsp_lword         owner;
	hfsp_lword         group;
	hfsp_lword         mode;
	hfsp_lword         dev;
} hfsplus_perm;

/* A single contiguous area of a file */
typedef struct {
	hfsp_lword         start_block;
	hfsp_lword         block_count;
} hfsplus_extent;
typedef hfsplus_extent hfsplus_extent_rec[8];

/* Information for a "Fork" in a file */
typedef struct {
	hfsp_llword        total_size;
	hfsp_lword         clump_size;
	hfsp_lword         total_blocks;
	hfsplus_extent_rec extents;
} hfsplus_fork_raw;

/* HFS+ Volume Header */
typedef struct hfsplus_vh{
	hfsp_word          signature;
	hfsp_word          version;
	hfsp_lword         attributes;
	hfsp_lword         last_mount_vers;
	hfsp_lword         reserved;

	hfsp_lword         create_date;
	hfsp_lword         modify_date;
	hfsp_lword         backup_date;
	hfsp_lword         checked_date;

	hfsp_lword         file_count;
	hfsp_lword         folder_count;

	hfsp_lword         blocksize;
	hfsp_lword         total_blocks;
	hfsp_lword         free_blocks;

	hfsp_lword         next_alloc;
	hfsp_lword         rsrc_clump_sz;
	hfsp_lword         data_clump_sz;
	hfsplus_cnid       next_cnid;

	hfsp_lword         write_count;
	hfsp_llword        encodings_bmp;

	hfsp_byte          finder_info[32];

	hfsplus_fork_raw   alloc_file;
	hfsplus_fork_raw   ext_file;
	hfsplus_fork_raw   cat_file;
	hfsplus_fork_raw   attr_file;
	hfsplus_fork_raw   start_file;
} hfsplus_vh;

/* HFS+ volume attributes */
#define HFSPLUS_VOL_UNMNT     (1 << 8)
#define HFSPLUS_VOL_SPARE_BLK (1 << 9)
#define HFSPLUS_VOL_NOCACHE   (1 << 10)
#define HFSPLUS_VOL_INCNSTNT  (1 << 11)
#define HFSPLUS_VOL_SOFTLOCK  (1 << 15)

/* HFS+ BTree node descriptor */
typedef struct {
	hfsp_lword         next;
	hfsp_lword         prev;
	hfsp_byte          kind;
	hfsp_byte          height;
	hfsp_word          num_rec;
	hfsp_word          reserved;
} hfsplus_btree_node_desc;

/* HFS+ BTree node types */
#define HFSPLUS_NODE_NDX  0x00
#define HFSPLUS_NODE_HEAD 0x01
#define HFSPLUS_NODE_MAP  0x02
#define HFSPLUS_NODE_LEAF 0xFF

/* HFS+ BTree header */
typedef struct {
	hfsp_word          depth;
	hfsp_lword         root;
	hfsp_lword         leaf_count;
	hfsp_lword         leaf_head;
	hfsp_lword         leaf_tail;
	hfsp_word          node_size;
	hfsp_word          max_key_len;
	hfsp_lword         node_count;
	hfsp_lword         free_nodes;
	hfsp_word          reserved1;
	hfsp_lword         clump_size;
	hfsp_byte          btree_type;
	hfsp_byte          reserved2;
	hfsp_lword         attributes;
	hfsp_lword         reserved3[16];
} hfsplus_btree_head;

/* BTree attributes */
#define HFSPLUS_TREE_BIGKEYS         2
#define HFSPLUS_TREE_VAR_NDXKEY_SIZE 4

/* HFS+ BTree misc info */
#define HFSPLUS_TREE_HEAD 0
#define HFSPLUS_NODE_MXSZ 32768

/* Some special File ID numbers (stolen from hfs.h) */
#define HFSPLUS_POR_CNID             1  /* Parent Of the Root */
#define HFSPLUS_ROOT_CNID            2  /* ROOT directory */
#define HFSPLUS_EXT_CNID             3  /* EXTents B-tree */
#define HFSPLUS_CAT_CNID             4  /* CATalog B-tree */
#define HFSPLUS_BAD_CNID             5  /* BAD blocks file */
#define HFSPLUS_ALLOC_CNID           6  /* ALLOCation file */
#define HFSPLUS_START_CNID           7  /* STARTup file */
#define HFSPLUS_ATTR_CNID            8  /* ATTRibutes file  */
#define HFSPLUS_EXCH_CNID           15  /* ExchangeFiles temp id */

/* HFS+ catalog entry key */
typedef struct {
	hfsplus_cnid       parent;
	hfsplus_unistr     name;
} hfsplus_cat_key;


/* Structs from hfs.h */
typedef struct {
        hfsp_word          v;
        hfsp_word          h;
} hfsp_point;

typedef struct {
        hfsp_word          top;
        hfsp_word          left;
        hfsp_word          bottom;
        hfsp_word          right;
} hfsp_rect;


/* HFS directory info (stolen from hfs.h */
typedef struct {
        hfsp_rect          frRect;
        hfsp_word          frFlags;
        hfsp_point         frLocation;
        hfsp_word          frView;
} DInfo;

typedef struct {
        hfsp_point         frScroll;
        hfsp_lword         frOpenChain;
        hfsp_word          frUnused;
        hfsp_word          frComment;
        hfsp_lword         frPutAway;
} DXInfo;

/* HFS+ folder data (part of an hfsplus_cat_entry) */
typedef struct {
	hfsp_word          flags;
	hfsp_lword         valence;
	hfsplus_cnid       id;
	hfsp_lword         create_date;
	hfsp_lword         content_mod_date;
	hfsp_lword         attribute_mod_date;
	hfsp_lword         access_date;
	hfsp_lword         backup_date;
	hfsplus_perm       permissions;
	DInfo              user_info;
	DXInfo             finder_info;
	hfsp_lword         text_encoding;
	hfsp_lword         reserved;
} hfsplus_cat_folder;

/* HFS file info (stolen from hfs.h) */
typedef struct {
        hfsp_lword         fdType;
        hfsp_lword         fdCreator;
        hfsp_word          fdFlags;
        hfsp_point         fdLocation;
        hfsp_word          fdFldr;
} FInfo;

typedef struct {
        hfsp_word          fdIconID;
        hfsp_byte          fdUnused[8];
        hfsp_word          fdComment;
        hfsp_lword         fdPutAway;
} FXInfo;

/* HFS+ file data (part of a cat_entry) */
typedef struct {
	hfsp_word          flags;
	hfsp_lword         reserved1;
	hfsplus_cnid       id;
	hfsp_lword         create_date;
	hfsp_lword         content_mod_date;
	hfsp_lword         attribute_mod_date;
	hfsp_lword         access_date;
	hfsp_lword         backup_date;
	hfsplus_perm       permissions;
	FInfo              user_info;
	FXInfo             finder_info;
	hfsp_lword         text_encoding;
	hfsp_lword         reserved2;

	hfsplus_fork_raw   data_fork;
	hfsplus_fork_raw   res_fork;
} hfsplus_cat_file;

/* File attribute bits */
#define kHFSFileLockedBit       0x0000
#define kHFSFileLockedMask      0x0001
#define kHFSThreadExistsBit     0x0001
#define kHFSThreadExistsMask    0x0002

/* HFS+ catalog thread (part of a cat_entry) */
typedef struct {
	hfsp_word          reserved;
	hfsplus_cnid       parentID;
	hfsplus_unistr     nodeName;
} hfsplus_cat_thread;

#define HFSPLUS_MIN_THREAD_SZ 10

/* A data record in the catalog tree */
typedef struct {
	hfsp_word          type;
	union {
		hfsplus_cat_folder folder;
		hfsplus_cat_file   file;
		hfsplus_cat_thread thread;
	} u;
} hfsplus_cat_entry;

/* HFS+ catalog entry type */
#define HFSPLUS_FOLDER         0x0001
#define HFSPLUS_FILE           0x0002
#define HFSPLUS_FOLDER_THREAD  0x0003
#define HFSPLUS_FILE_THREAD    0x0004

/* HFS+ extents tree key */
typedef struct {
	hfsp_byte          fork_type;
	hfsp_byte          pad;
	hfsplus_cnid       cnid;
	hfsp_lword         start_block;
} hfsplus_ext_key;

#define HFSPLUS_EXT_KEYLEN 10

/* HFS+ generic BTree key */
typedef struct {
	hfsp_word          key_len;
	union {
		hfsplus_cat_key cat;
		hfsplus_ext_key ext;
	} u;
} hfsplus_btree_key;

#endif
